"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.emitOne = exports.emitMany = void 0;
const Keywords = require("../tokens/keywords");
const Delimiters = require("../tokens/delimiters");
const GenericTokens = require("../tokens/generic_tokens");
const ScopeTokens = require("../tokens/scope_tokens");
const TypeReferenceTokens = require("../tokens/type_reference_tokens");
const AttributeEmitter = require("./attribute_emitter");
const FieldEmitter = require("./field_emitter");
const PropertyEmitter = require("./property_emitter");
const MethodEmitter = require("./method_emitter");
function emitMany(writer, classes) {
    if (classes == null || classes.length === 0)
        return;
    classes.forEach((c, i, arr) => emitOne(writer, c, i === arr.length - 1));
}
exports.emitMany = emitMany;
function emitOne(writer, _class, isLast) {
    emitAttributes(writer, _class);
    emitDefinition(writer, _class);
    emitFields(writer, _class);
    emitProperties(writer, _class);
    emitMethods(writer, _class);
    emitEnd(writer, isLast);
}
exports.emitOne = emitOne;
function emitDefinition(writer, _class) {
    const { access, isPartial, modifier, name, generics, baseClass } = _class;
    const accessTokens = access ? [access, Delimiters.space] : [];
    const partialTokens = isPartial ? [Keywords.partial, Delimiters.space] : [];
    const modifierTokens = modifier ? [modifier, Delimiters.space] : [];
    const genericTokens = GenericTokens.emit(generics);
    const baseClassTokens = baseClass ? TypeReferenceTokens.emit(baseClass) : [];
    const extensionTokens = baseClassTokens.length ? [Delimiters.space, Delimiters.colon, Delimiters.space, ...baseClassTokens] : [];
    writer
        .outputTabs()
        .write(...accessTokens)
        .write(...modifierTokens)
        .write(...partialTokens)
        .write(Keywords._class, Delimiters.space, name)
        .write(...genericTokens)
        .write(...extensionTokens)
        .writeLineNoTabs()
        .writeLine(ScopeTokens.open)
        .indent();
}
function emitAttributes(writer, _class) {
    const { attributes } = _class;
    AttributeEmitter.emitMany(writer, attributes);
}
function emitFields(writer, _class) {
    const { fields } = _class;
    FieldEmitter.emitMany(writer, fields, true);
}
function emitProperties(writer, _class) {
    const { properties, fields } = _class;
    PropertyEmitter.emitMany(writer, properties, (fields == null || fields.length === 1));
}
function emitMethods(writer, _class) {
    const { methods, properties } = _class;
    MethodEmitter.emitMany(writer, methods, (properties == null || properties.length === 1));
}
function emitEnd(writer, isLast) {
    writer
        .unindent()
        .writeLine(ScopeTokens.close);
    if (!isLast)
        writer.writeLine();
}
